{ Demonstration program for ChroMathsome - Statistics.

  This demo program shows the use of more advanced statmatrix functions of Chromathsome - Statistics.

  The reader may do with this code whatever he/she wants.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: June '98

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  ChroMathsome - Statistics 1996-98 John Merrit. All Rights Reserved

  Brought to you by the letter "M".             }

unit UMatrix;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, Statistc, StdCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    InvertStatMatrix1: TMenuItem;
    Determinant1: TMenuItem;
    Memo1: TMemo;
    SysOfEquations1: TMenuItem;
    procedure Close1Click(Sender: TObject);
    procedure InvertStatMatrix1Click(Sender: TObject);
    procedure SysOfEquations1Click(Sender: TObject);
    procedure Determinant1Click(Sender: TObject);
  private
    { Private declarations }
    procedure Write_StatMatrix(M: StatMatrix);
    procedure Write_StatVector(V: StatVector);
    
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

{ Write a statmatrix to the screen, creating a nice readable look. }
procedure TForm1.Write_StatMatrix(M: StatMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-18 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Write a Stat vector to the screen, creating a nice readable look. }
procedure TForm1.Write_StatVector(V: StatVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if abs(V.Vector^[a]) < 1E-17 then
    begin
    V.Vector^[a]:= 0;
    matt:= IntToStr(round(V.Vector^[a]));
    end
    else
    matt:= FloatToStrF(V.Vector^[a], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

{ Invert a StatMatrix. }
procedure TForm1.InvertStatMatrix1Click(Sender: TObject);
var M: StatMatrix;
begin
     InitStatMatrix(M, 3, 3);

     try

     M.Matrix^[1]^[1]:= 1;
     M.Matrix^[1]^[2]:= 3;
     M.Matrix^[1]^[3]:= 3;

     M.Matrix^[2]^[1]:= 6;
     M.Matrix^[2]^[2]:= 5;
     M.Matrix^[2]^[3]:= 3;

     M.Matrix^[3]^[1]:= 88;
     M.Matrix^[3]^[2]:= 97;
     M.Matrix^[3]^[3]:= 3;

     Memo1.Lines.Add('Original matrix: ');
     Write_StatMatrix(M);
     StatMatrixInverse(M);
     Memo1.Lines.Add('Inverse matrix: ');
     Write_StatMatrix(M);

     StatMatrixInverse(M);
     Memo1.Lines.Add('Inverse matrix: ');
     Write_StatMatrix(M);

     finally
     DestroyStatMatrix(M);
     end;
end;

{ Solve some systems of equations. }
procedure TForm1.SysOfEquations1Click(Sender: TObject);
var CoV: StatVector;
    NumOfSolutions: integer;
    M, CoM: StatMatrix;
begin
     InitStatMatrix(M, 3, 3);
     InitStatMatrix(CoM, 3, 3);
     InitStatVector(CoV, 3);

     try
     { Example 1: Simple 3 * 3 system -> infinitely many solutions besides the trivial one (0, 0, 0). }
     CoM.Matrix^[1]^[1]:= -18;
     CoM.Matrix^[1]^[2]:= -57;
     CoM.Matrix^[1]^[3]:= -69;

     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 4;
     CoM.Matrix^[2]^[3]:= 3;

     CoM.Matrix^[3]^[1]:= 5;
     CoM.Matrix^[3]^[2]:= 15;
     CoM.Matrix^[3]^[3]:= 20;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;

     Memo1.Lines.Add('System A * X = Y with: ');
     Memo1.Lines.Add('A: ');
     Write_StatMatrix(CoM);
     Memo1.Lines.Add('Y: ');
     Write_StatVector(CoV);

     StatSysOfEquations(M, CoM, CoV, NumOfSolutions);
     Memo1.Lines.Add('Solutions of system: ');
     Write_StatMatrix(M);

     { Example 2: 6 * 6 system -> one solution. }
     RedimensionStatMatrix(CoM, 6, 6);
     RedimensionStatVector(CoV, 6);
     CoM.Matrix^[1]^[1]:= 0;
     CoM.Matrix^[1]^[2]:= 0;
     CoM.Matrix^[1]^[3]:= 0;
     CoM.Matrix^[1]^[4]:= 1;
     CoM.Matrix^[1]^[5]:= -1;
     CoM.Matrix^[1]^[6]:= 1;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -1;
     CoM.Matrix^[2]^[3]:= 0;
     CoM.Matrix^[2]^[4]:= 0;
     CoM.Matrix^[2]^[5]:= 1;
     CoM.Matrix^[2]^[6]:= 0;
     CoM.Matrix^[3]^[1]:= -1;
     CoM.Matrix^[3]^[2]:= 0;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[3]^[4]:= 0;
     CoM.Matrix^[3]^[5]:= 0;
     CoM.Matrix^[3]^[6]:= -1;
     CoM.Matrix^[4]^[1]:= 0;
     CoM.Matrix^[4]^[2]:= -2;
     CoM.Matrix^[4]^[3]:= 0;
     CoM.Matrix^[4]^[4]:= -3;
     CoM.Matrix^[4]^[5]:= -4;
     CoM.Matrix^[4]^[6]:= 0;
     CoM.Matrix^[5]^[1]:= 4;
     CoM.Matrix^[5]^[2]:= 2;
     CoM.Matrix^[5]^[3]:= 3;
     CoM.Matrix^[5]^[4]:= 0;
     CoM.Matrix^[5]^[5]:= 0;
     CoM.Matrix^[5]^[6]:= 0;
     CoM.Matrix^[6]^[1]:= -4;
     CoM.Matrix^[6]^[2]:= 0;
     CoM.Matrix^[6]^[3]:= 0;
     CoM.Matrix^[6]^[4]:= 0;
     CoM.Matrix^[6]^[5]:= 4;
     CoM.Matrix^[6]^[6]:= 6;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     CoV.Vector^[4]:= 6;
     CoV.Vector^[5]:= -2;
     CoV.Vector^[6]:= 20;

     Memo1.Lines.Add('System A * X = Y with: ');
     Memo1.Lines.Add('A: ');
     Write_StatMatrix(CoM);
     Memo1.Lines.Add('Y: ');
     Write_StatVector(CoV);

     StatSysOfEquations(M, CoM, CoV, NumOfSolutions);
     Memo1.Lines.Add('Solutions of system: ');
     Write_StatMatrix(M);

     { Example 3: 4 * 5 system -> infinitely many solutions. }
     RedimensionStatMatrix(CoM, 4, 5);
     RedimensionStatVector(CoV, 4);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[1]^[3]:= -2;
     CoM.Matrix^[1]^[4]:= -1;
     CoM.Matrix^[1]^[5]:= 0;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 3;
     CoM.Matrix^[2]^[3]:= -5;
     CoM.Matrix^[2]^[4]:= 1;
     CoM.Matrix^[2]^[5]:= -7;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[3]^[4]:= -1;
     CoM.Matrix^[3]^[5]:= 1;
     CoM.Matrix^[4]^[1]:= -1;
     CoM.Matrix^[4]^[2]:= 1;
     CoM.Matrix^[4]^[3]:= 5;
     CoM.Matrix^[4]^[4]:= -1;
     CoM.Matrix^[4]^[5]:= 0;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 4;
     CoV.Vector^[3]:= 2;
     CoV.Vector^[4]:= 3;

     Memo1.Lines.Add('System A * X = Y with: ');
     Memo1.Lines.Add('A: ');
     Write_StatMatrix(CoM);
     Memo1.Lines.Add('Y: ');
     Write_StatVector(CoV);

     StatSysOfEquations(M, CoM, CoV, NumOfSolutions);
     Memo1.Lines.Add('Solutions of system: ');
     Write_StatMatrix(M);

     { Example 4: 3 * 4 system -> infinitely many solutions. }
     RedimensionStatMatrix(CoM, 3, 4);
     RedimensionStatVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= -1;
     CoM.Matrix^[1]^[3]:= 5;
     CoM.Matrix^[1]^[4]:= -1;
     CoM.Matrix^[2]^[1]:= -2;
     CoM.Matrix^[2]^[2]:= 1;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[2]^[4]:= -1;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 2;
     CoM.Matrix^[3]^[3]:= -1;
     CoM.Matrix^[3]^[4]:= -1;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= -5;
     CoV.Vector^[3]:= -2;

     Memo1.Lines.Add('System A * X = Y with: ');
     Memo1.Lines.Add('A: ');
     Write_StatMatrix(CoM);
     Memo1.Lines.Add('Y: ');
     Write_StatVector(CoV);

     StatSysOfEquations(M, CoM, CoV, NumOfSolutions);
     Memo1.Lines.Add('Solutions of system: ');
     Write_StatMatrix(M);

     finally
     DestroyStatMatrix(M);
     DestroyStatMatrix(CoM);
     DestroyStatVector(CoV);
     end;
end;

procedure TForm1.Determinant1Click(Sender: TObject);
var M: StatMatrix;
    x: extended;
begin
     InitStatMatrix(M, 3, 3);

     try

     M.Matrix^[1]^[1]:= 1;
     M.Matrix^[1]^[2]:= 3;
     M.Matrix^[1]^[3]:= 3;

     M.Matrix^[2]^[1]:= 6;
     M.Matrix^[2]^[2]:= 5;
     M.Matrix^[2]^[3]:= 3;

     M.Matrix^[3]^[1]:= 88;
     M.Matrix^[3]^[2]:= 97;
     M.Matrix^[3]^[3]:= 3;

     Memo1.Lines.Add('Original matrix: ');
     Write_StatMatrix(M);
     x:= StatMatrixDeterminant(M);
     Memo1.Lines.Add('Determinant of matrix: ' + FloatToStr(x));

     finally
     DestroyStatMatrix(M);
     end;
end;

end.
